/* --------------------------------------------------------------------------
 *
 * ETK++ --- The Easy Toolkit for C++ programing
 * Copyright (C) 2004-2006, Anthony Lee, All Rights Reserved
 *
 * $COPYING
 *
 * File: Application.h
 * Description: Application model to support Looper/Message
 *
 * --------------------------------------------------------------------------*/

#ifndef __ETK_APPLICATION_H__
#define __ETK_APPLICATION_H__

#include <etk/app/Looper.h>
#include <etk/app/MessageRunner.h>
#include <etk/app/Cursor.h>

#ifdef __cplusplus /* Just for C++ */


class EClipboard;
class EGraphicsEngine;


class _IMPEXP_ETK EApplication : public ELooper {
public:
	EApplication(const char *signature, bool tryInterface = true);
	virtual ~EApplication();

	// Archiving
	EApplication(EMessage *from);
	virtual e_status_t Archive(EMessage *into, bool deep = true) const;
	static EArchivable *Instantiate(EMessage *from);

	const char		*Signature() const;

	virtual void		*Run();
	virtual void		Quit();
	virtual bool		QuitRequested();

	// Empty functions BEGIN --- just for derivative class
	virtual void		ReadyToRun();
	virtual void		Pulse();
	// Empty functions END

	void			SetPulseRate(e_bigtime_t rate);
	e_bigtime_t		PulseRate() const;

	virtual void		MessageReceived(EMessage *msg);
	virtual void		DispatchMessage(EMessage *msg, EHandler *target);

	void			SetCursor(const void *cursor);
	void			SetCursor(const ECursor *cursor, bool sync = true);
	void			HideCursor();
	void			ShowCursor();
	void			ObscureCursor();
	bool			IsCursorHidden() const;

private:
	friend class ELooper;
	friend class EMessageRunner;
	friend class EWindow;
	friend class EView;
	friend class EAlert;
	friend class EBitmap;

	friend _IMPEXP_ETK bool etk_update_font_families(bool);

	bool _quit_;
	char *fSignature;
	e_bigtime_t fPulseRate;
	EMessageRunner *fPulseRunner;

	static EList sRunnerList;
	static e_bigtime_t sRunnerMinimumInterval;
	static void etk_dispatch_message_runners();

	bool etk_quit_all_loopers(bool force);

	EGraphicsEngine *fGraphicsEngine;
	void *fGraphicsEngineAddon;
	void InitGraphicsEngine();

	void Init(const char *signature, bool tryInterface);

	EList fModalWindows;
	bool AddModalWindow(EMessenger &msgr);
	bool RemoveModalWindow(EMessenger &msgr);

	ECursor fCursor;
	bool fCursorHidden;
	bool fCursorObscure;
};


inline void EApplication::SetCursor(const void *cursor)
{
	ECursor theCursor(cursor);
	SetCursor(&theCursor, true);
}


extern _IMPEXP_ETK EApplication *etk_app;
extern _IMPEXP_ETK EMessenger etk_app_messenger;
extern _IMPEXP_ETK EClipboard etk_clipboard;

#endif /* __cplusplus */

#endif /* __ETK_APPLICATION_H__ */

