/* --------------------------------------------------------------------------
 *
 * ETK++ --- The Easy Toolkit for C++ programing
 * Copyright (C) 2004-2006, Anthony Lee, All Rights Reserved
 *
 * $COPYING
 *
 * File: Message.h
 * Description: Message for communication between loopers/teams
 *
 * --------------------------------------------------------------------------*/

#ifndef __ETK_MESSAGE_H__
#define __ETK_MESSAGE_H__

#include <etk/support/SupportDefs.h>
#include <etk/support/String.h>
#include <etk/support/List.h>
#include <etk/interface/Point.h>
#include <etk/interface/Rect.h>
#include <etk/kernel/OS.h>

#ifdef __cplusplus /* Just for C++ */

class EMessenger;
class EHandler;

class _IMPEXP_ETK EMessage {
public:
	EMessage();
	EMessage(euint32 what);
	EMessage(const EMessage &msg);
	virtual ~EMessage();

	euint32		what;

	EMessage	&operator=(const EMessage &msg);

	eint32		CountTypesByName(const char *name) const;
	bool		TypeAt(const char *name, eint32 index, e_type_code *type) const;
	eint32		CountItems(const char *name, e_type_code type) const;

	void		MakeEmpty();
	bool		IsEmpty() const;

	bool		Rename(const char *old_entry, const char *new_entry);

	bool		AddString(const char *name, const char *aString);
	bool		AddString(const char *name, const EString &aString);
	bool		AddInt8(const char *name, eint8 val);
	bool		AddInt16(const char *name, eint16 val);
	bool		AddInt32(const char *name, eint32 val);
	bool		AddInt64(const char *name, eint64 val);
	bool		AddBool(const char *name, bool aBoolean);
	bool		AddFloat(const char *name, float aFloat);
	bool		AddDouble(const char *name, double aDouble);
	bool		AddPoint(const char *name, EPoint pt);
	bool		AddRect(const char *name, ERect r);
	bool		AddPointer(const char *name, const void *ptr);
	bool		AddMessage(const char *name, const EMessage *msg);
	bool		AddMessenger(const char *name, const EMessenger *msgr);
	bool		AddMessenger(const char *name, const EMessenger &msgr);
	bool		AddData(const char *name, e_type_code type, const void *data, size_t numBytes, bool is_fixed_size = true);

	bool		FindString(const char *name, eint32 index, const char **str) const;
	bool		FindString(const char *name, const char **str) const;
	bool		FindString(const char *name, eint32 index, EString *str) const;
	bool		FindString(const char *name, EString *str) const;
	bool		FindInt8(const char *name, eint8 *val) const;
	bool		FindInt8(const char *name, eint32 index, eint8 *val) const;
	bool		FindInt16(const char *name, eint16 *val) const;
	bool		FindInt16(const char *name, eint32 index, eint16 *val) const;
	bool		FindInt32(const char *name, eint32 *val) const;
	bool		FindInt32(const char *name, eint32 index, eint32 *val) const;
	bool		FindInt64(const char *name, eint64 *val) const;
	bool		FindInt64(const char *name, eint32 index, eint64 *val) const;
	bool		FindBool(const char *name, bool *aBoolean) const;
	bool		FindBool(const char *name, eint32 index, bool *aBoolean) const;
	bool		FindFloat(const char *name, float *f) const;
	bool		FindFloat(const char *name, eint32 index, float *f) const;
	bool		FindDouble(const char *name, double *d) const;
	bool		FindDouble(const char *name, eint32 index, double *d) const;
	bool		FindPoint(const char *name, EPoint *pt) const;
	bool		FindPoint(const char *name, eint32 index, EPoint *pt) const;
	bool		FindRect(const char *name, ERect *r) const;
	bool		FindRect(const char *name, eint32 index, ERect *r) const;
	bool		FindPointer(const char *name, void **ptr) const;
	bool		FindPointer(const char *name, eint32 index, void **ptr) const;
	bool		FindMessage(const char *name, EMessage *msg) const;
	bool		FindMessage(const char *name, eint32 index, EMessage *msg) const;
	bool		FindMessenger(const char *name, EMessenger *msgr) const;
	bool		FindMessenger(const char *name, eint32 index, EMessenger *msgr) const;
	bool		FindData(const char *name, e_type_code type, const void **data, ssize_t *numBytes) const;
	bool		FindData(const char *name, e_type_code type, eint32 index, const void **data, ssize_t *numBytes) const;

	bool		HasString(const char *name, eint32 index = 0) const;
	bool		HasInt8(const char *name, eint32 index = 0) const;
	bool		HasInt16(const char *name, eint32 index = 0) const;
	bool		HasInt32(const char *name, eint32 index = 0) const;
	bool		HasInt64(const char *name, eint32 index = 0) const;
	bool		HasBool(const char *name, eint32 index = 0) const;
	bool		HasFloat(const char *name, eint32 index = 0) const;
	bool		HasDouble(const char *name, eint32 index = 0) const;
	bool		HasPoint(const char *name, eint32 index = 0) const;
	bool		HasRect(const char *name, eint32 index = 0) const;
	bool		HasPointer(const char *name, eint32 index = 0) const;
	bool		HasMessage(const char *name, eint32 index = 0) const;
	bool		HasMessenger(const char *name, eint32 index = 0) const;
	bool		HasData(const char *name, e_type_code type, eint32 index = 0) const;

	bool		RemoveString(const char *name, eint32 index = 0);
	bool		RemoveInt8(const char *name, eint32 index = 0);
	bool		RemoveInt16(const char *name, eint32 index = 0);
	bool		RemoveInt32(const char *name, eint32 index = 0);
	bool		RemoveInt64(const char *name, eint32 index = 0);
	bool		RemoveBool(const char *name, eint32 index = 0);
	bool		RemoveFloat(const char *name, eint32 index = 0);
	bool		RemoveDouble(const char *name, eint32 index = 0);
	bool		RemovePoint(const char *name, eint32 index = 0);
	bool		RemoveRect(const char *name, eint32 index = 0);
	bool		RemovePointer(const char *name, eint32 index = 0);
	bool		RemoveMessage(const char *name, eint32 index = 0);
	bool		RemoveMessenger(const char *name, eint32 index = 0);
	bool		RemoveData(const char *name, e_type_code type, eint32 index = 0);
	bool		RemoveData(const char *name, e_type_code type);
	bool		RemoveData(const char *name);

	bool		ReplaceString(const char *name, eint32 index, const char *aString);
	bool		ReplaceString(const char *name, const char *aString);
	bool		ReplaceString(const char *name, eint32 index, const EString &aString);
	bool		ReplaceString(const char *name, const EString &aString);
	bool		ReplaceInt8(const char *name, eint8 val);
	bool		ReplaceInt8(const char *name, eint32 index, eint8 val);
	bool		ReplaceInt16(const char *name, eint16 val);
	bool		ReplaceInt16(const char *name, eint32 index, eint16 val);
	bool		ReplaceInt32(const char *name, eint32 val);
	bool		ReplaceInt32(const char *name, eint32 index, eint32 val);
	bool		ReplaceInt64(const char *name, eint64 val);
	bool		ReplaceInt64(const char *name, eint32 index, eint64 val);
	bool		ReplaceBool(const char *name, bool aBoolean);
	bool		ReplaceBool(const char *name, eint32 index, bool aBoolean);
	bool		ReplaceFloat(const char *name, float f);
	bool		ReplaceFloat(const char *name, eint32 index, float f);
	bool		ReplaceDouble(const char *name, double d);
	bool		ReplaceDouble(const char *name, eint32 index, double d);
	bool		ReplacePoint(const char *name, EPoint pt);
	bool		ReplacePoint(const char *name, eint32 index, EPoint pt);
	bool		ReplaceRect(const char *name, ERect r);
	bool		ReplaceRect(const char *name, eint32 index, ERect r);
	bool		ReplacePointer(const char *name, const void *ptr);
	bool		ReplacePointer(const char *name, eint32 index, const void *ptr);
	bool		ReplaceMessage(const char *name, const EMessage *msg);
	bool		ReplaceMessage(const char *name, eint32 index, const EMessage *msg);
	bool		ReplaceMessenger(const char *name, const EMessenger *msgr);
	bool		ReplaceMessenger(const char *name, eint32 index, const EMessenger *msgr);
	bool		ReplaceMessenger(const char *name, const EMessenger &msgr);
	bool		ReplaceMessenger(const char *name, eint32 index, const EMessenger &msgr);
	bool		ReplaceData(const char *name, e_type_code type, const void *data, size_t numBytes, bool is_fixed_size);
	bool		ReplaceData(const char *name, e_type_code type, eint32 index, const void *data, size_t numBytes, bool is_fixed_size);

	void		PrintToStream() const;

	size_t		FlattenedSize() const;
	bool		Flatten(char *buffer, size_t bufferSize) const;
	bool		Unflatten(const char *buffer, size_t bufferSize);

	bool		WasDelivered() const;
	bool		IsReply() const;
	bool		IsSourceWaiting() const;

	e_status_t	SendReply(euint32 command, EHandler *replyHandler = NULL) const;
	e_status_t	SendReply(const EMessage *message,
				  EHandler *replyHandler = NULL,
				  e_bigtime_t sendTimeout = E_INFINITE_TIMEOUT) const;

private:
	friend class ELooper;
	friend class EMessenger;

	typedef struct list_data {
		char 		*name;
		EList		list;
	} list_data;

	typedef struct type_list_data {
		e_type_code	type;
		EList		list;
	} type_list_data;

	typedef struct _object_ {
		size_t		bytes;
		bool		fixed_size;
		void		*data;
	} _object_;

	EList ObjectsList;

	static list_data *_find_list_(const EList *list, const char *name);
	static type_list_data *_find_type_list_(const EList *list, e_type_code type);

	eint64 fTeam;

	euint64 fTargetToken;
	e_bigtime_t fTargetTokenTimestamp;
	euint64 fReplyToken;
	e_bigtime_t fReplyTokenTimestamp;

	bool fNoticeSource;
	void *fSource;

	bool fIsReply;
};

#endif /* __cplusplus */

#endif /* __ETK_MESSAGE_H__ */


