/* --------------------------------------------------------------------------
 *
 * ETK++ --- The Easy Toolkit for C++ programing
 * Copyright (C) 2004-2006, Anthony Lee, All Rights Reserved
 *
 * $COPYING
 *
 * File: MessageFilter.h
 * Description: Filter message before ELooper::DispatchMessage
 * Warning: ignore E_QUIT_REQUESTED & _QUIT_
 *
 * --------------------------------------------------------------------------*/

#ifndef __ETK_MESSAGE_FILTER_H__
#define __ETK_MESSAGE_FILTER_H__

#include <etk/app/Handler.h>

typedef enum e_filter_result {
	E_SKIP_MESSAGE,
	E_DISPATCH_MESSAGE
} e_filter_result;

typedef enum e_message_delivery {
	E_ANY_DELIVERY,
	E_DROPPED_DELIVERY,
	E_PROGRAMMED_DELIVERY
} e_message_delivery;

typedef enum e_message_source {
	E_ANY_SOURCE,
	E_REMOTE_SOURCE,
	E_LOCAL_SOURCE
} e_message_source;

#ifdef __cplusplus /* Just for C++ */

class EMessage;
class EMessageFilter;

typedef e_filter_result (*e_filter_hook)(EMessage *message, EHandler **target, EMessageFilter *filter);


class _IMPEXP_ETK EMessageFilter {
public:
	EMessageFilter(e_message_delivery delivery, e_message_source source,
		       euint32 command, e_filter_hook filter = NULL);
	EMessageFilter(e_message_delivery delivery, e_message_source source,
		       e_filter_hook filter = NULL);
	EMessageFilter(euint32 command, e_filter_hook filter = NULL);
	EMessageFilter(const EMessageFilter &filter);
	EMessageFilter(const EMessageFilter *filter);
	virtual ~EMessageFilter();

	EMessageFilter &operator=(const EMessageFilter &from);

	virtual e_filter_result		Filter(EMessage *message, EHandler **target);

	e_message_delivery		MessageDelivery() const;
	e_message_source		MessageSource() const;
	euint32				Command() const;
	bool				FiltersAnyCommand() const;
	ELooper				*Looper() const;

private:
	friend class ELooper;
	friend class EHandler;

	euint32 fCommand;
	bool fFiltersAny;
	e_message_delivery fDelivery;
	e_message_source fSource;
	e_filter_hook fFilterHook;

	ELooper *fLooper;
	EHandler *fHandler;

	e_filter_result doFilter(EMessage *message, EHandler **target);
};

#endif /* __cplusplus */

#endif /* __ETK_MESSAGE_FILTER_H__ */

