/* --------------------------------------------------------------------------
 *
 * ETK++ --- The Easy Toolkit for C++ programing
 * Copyright (C) 2004-2006, Anthony Lee, All Rights Reserved
 *
 * $COPYING
 *
 * File: MessageRunner.h
 * Description: Sending messages at intervals
 *
 * --------------------------------------------------------------------------*/

#ifndef __ETK_MESSAGE_RUNNER_H__
#define __ETK_MESSAGE_RUNNER_H__

#include <etk/app/Messenger.h>

#ifdef __cplusplus /* Just for C++ */

class _IMPEXP_ETK EMessageRunner {
public:
	EMessageRunner(const EMessenger &target,
		       const EMessage *msg,
		       e_bigtime_t interval,
		       eint32 count = -1);
	EMessageRunner(const EMessenger &target,
		       const EMessage *msg,
		       e_bigtime_t interval,
		       eint32 count,
		       const EMessenger &replyTo);
	virtual ~EMessageRunner();

	bool		IsValid() const;

	e_status_t	SetTarget(const EMessenger &target);
	e_status_t	SetReplyTo(const EMessenger &replyTo);
	e_status_t	SetMessage(const EMessage *msg);
	e_status_t	SetInterval(e_bigtime_t interval);
	e_status_t	SetCount(eint32 count);

	e_status_t	GetInfo(e_bigtime_t *interval, eint32 *count) const;
	e_status_t	GetInfo(EMessenger *target, EMessage *msg,
				e_bigtime_t *interval, eint32 *count,
				EMessenger *replyTo = NULL) const;

private:
	friend class EApplication;

	eint32 fToken;

	EMessenger *fTarget;
	EMessenger *fReplyTo;
	EMessage *fMessage;
	e_bigtime_t fInterval;
	e_bigtime_t fPrevSendTime;
	eint32 fCount;
};

#endif /* __cplusplus */

#endif /* __ETK_MESSAGE_RUNNER_H__ */

