/* --------------------------------------------------------------------------
 * 
 * ETK++ --- The Easy Toolkit for C++ programing
 * Copyright (C) 2004-2006, Anthony Lee, All Rights Reserved
 *
 * $COPYING
 * 
 * File: MenuField.h
 * Description: EMenuField --- display a labeled pop-up menu
 * 
 * --------------------------------------------------------------------------*/

#ifndef __ETK_MENU_FIELD_H__
#define __ETK_MENU_FIELD_H__

#include <etk/interface/MenuBar.h>

#ifdef __cplusplus /* Just for C++ */

class _IMPEXP_ETK EMenuField : public EView {
public:
	EMenuField(ERect frame,
		   const char *name,
		   const char *label,
		   EMenu *menu,
		   bool fixedSize = false,
		   euint32 resizeMode = E_FOLLOW_LEFT | E_FOLLOW_TOP,
		   euint32 flags = E_WILL_DRAW | E_NAVIGABLE);
	virtual ~EMenuField();

	virtual void	SetLabel(const char *label);
	const char	*Label() const;

	virtual	void	SetAlignment(e_alignment alignment);
	e_alignment	Alignment() const;

	virtual void	SetDivider(float divider);
	float		Divider() const;

	// SetMenu: the old menu will be destructed automatically.
	virtual bool	SetMenu(EMenu *menu);
	EMenu		*Menu() const;

	EMenuBar	*MenuBar() const;
	EMenuItem	*MenuItem() const;

	virtual	void	Draw(ERect updateRect);
	virtual	void	GetPreferredSize(float *width, float *height);
	virtual	void	ResizeToPreferred();
	virtual void	WindowActivated(bool state);
	virtual void	MakeFocus(bool focusState = true);
	virtual void	SetFont(const EFont *font, euint8 mask = E_FONT_ALL);
	virtual	void	MouseDown(EPoint where);
	virtual	void	MouseUp(EPoint where);
	virtual void	MouseMoved(EPoint where, euint32 code, const EMessage *a_message);
	virtual void	KeyDown(const char *bytes, eint32 numBytes);
	virtual void	KeyUp(const char *bytes, eint32 numBytes);

protected:
	virtual void	ChildRemoving(EView *child);

private:
	bool fFixedSize;
	e_alignment fAlignment;
	float fDivider;

	char *fLabel;
	EMenuBar *fMenuBar;
	EMenu *fMenu;
};

#endif /* __cplusplus */

#endif /* __ETK_MENU_FIELD_H__ */

