#ifndef __ETK_MENU_ITEM_H__
#define __ETK_MENU_ITEM_H__

#include <etk/support/Archivable.h>
#include <etk/app/Invoker.h>

#ifdef __cplusplus /* Just for C++ */

class EMenu;

class _IMPEXP_ETK EMenuItem : public EArchivable, public EInvoker {
public:
	EMenuItem(const char *label, EMessage *message, char shortcut = 0, euint32 modifiers = 0);
	EMenuItem(EMenu *menu, EMessage *message = NULL);
	virtual ~EMenuItem();

	// Archiving
	EMenuItem(EMessage *from);
	virtual e_status_t Archive(EMessage *into, bool deep = true) const;
	static EArchivable *Instantiate(EMessage *from);

	virtual void		SetLabel(const char *label);
	virtual void		SetEnabled(bool state);
	virtual void		SetMarked(bool state);
	virtual void		SetShortcut(char ch, euint32 modifiers);

	const char*		Label() const;
	bool			IsEnabled() const;
	bool			IsMarked() const;
	char			Shortcut(euint32 *modifiers = NULL) const;

	EMenu*			Submenu() const;
	EMenu*			Menu() const;
	ERect			Frame() const;

	virtual	e_status_t	Invoke(const EMessage *msg = NULL);

protected:
	friend class EMenu;

	bool		IsSelected() const;
	virtual bool	SelectChanged();

	virtual void	GetContentSize(float *width, float *height) const;

	virtual void	DrawContent();
	virtual void	Draw();
	virtual void	Highlight(bool on);

private:
	char fShortcut;
	euint32 fModifiers;

	ERect fFrame;
	bool fMarked;
	bool fEnabled;

	char *fLabel;
	char *fShortcuts;

	EMenu *fSubmenu;
	EMenu *fMenu;

	void ShowSubmenu(bool selectFirstItem = true);
};


class _IMPEXP_ETK EMenuSeparatorItem : public EMenuItem {
public:
	EMenuSeparatorItem();
	virtual ~EMenuSeparatorItem();

protected:
	virtual void	GetContentSize(float *width, float *height) const;
	virtual void	Draw();
	virtual bool	SelectChanged();
};


#endif /* __cplusplus */

#endif /* __ETK_MENU_ITEM_H__ */

