/* --------------------------------------------------------------------------
 * 
 * ETK++ --- The Easy Toolkit for C++ programing
 * Copyright (C) 2004-2006, Anthony Lee, All Rights Reserved
 * 
 * $COPYING
 *
 * File: Region.cpp
 * Description: ERegion --- Combination of rectangles to describe region
 * 
 * --------------------------------------------------------------------------*/

#ifndef __ETK_REGION_H__
#define __ETK_REGION_H__

#include <etk/support/List.h>
#include <etk/interface/Rect.h>

#ifdef __cplusplus /* Just for C++ */

class _IMPEXP_ETK ERegion {
public:
	ERegion();
	ERegion(const ERegion &region);
	ERegion(const ERect &rect);
	virtual ~ERegion();

	ERegion &operator=(const ERegion &from);

	ERegion operator&(ERect r) const;
	ERegion operator|(ERect r) const;

	ERegion& operator&=(ERect r);
	ERegion& operator|=(ERect r);

	ERegion operator&(const ERegion &region) const;
	ERegion operator|(const ERegion &region) const;

	ERegion& operator&=(const ERegion &region);
	ERegion& operator|=(const ERegion &region);

	ERect Frame() const;
	ERect RectAt(eint32 index) const;
	eint32 CountRects() const;

	void Set(ERect singleBound);
	void MakeEmpty();

	bool Include(ERect r);
	bool Include(const ERegion *region);

	bool Exclude(ERect r);
	bool Exclude(const ERegion *region);

	void OffsetBy(float dx, float dy);
	void OffsetBy(EPoint pt);
	ERegion& OffsetBySelf(float dx, float dy);
	ERegion& OffsetBySelf(EPoint pt);
	ERegion OffsetByCopy(float dx, float dy);
	ERegion OffsetByCopy(EPoint pt);

	void Scale(float scaling);
	ERegion& ScaleSelf(float scaling);
	ERegion ScaleCopy(float scaling);

	bool Intersect(ERect r) const;
	bool Intersect(float l, float t, float r, float b) const;
	bool Intersect(const ERegion *region) const;

	bool Contains(EPoint pt) const;
	bool Contains(float x, float y) const;
	bool Contains(ERect r) const;
	bool Contains(float l, float t, float r, float b) const;
	bool Contains(const ERegion *region) const;

	void PrintToStream() const;

private:
	EList fRects;
	ERect fFrame;
};

#endif /* __cplusplus */

#endif /* __ETK_REGION_H__ */

