#ifndef __ETK_SCROLL_BAR_H__
#define __ETK_SCROLL_BAR_H__

#include <etk/app/MessageRunner.h>
#include <etk/interface/View.h>

#define E_H_SCROLL_BAR_WIDTH	e_ui_get_scrollbar_horizontal_width()
#define E_V_SCROLL_BAR_HEIGHT	e_ui_get_scrollbar_vertical_height()

#ifdef __cplusplus /* Just for C++ */

class _IMPEXP_ETK EScrollBar : public EView {
public:
	EScrollBar(ERect frame, const char *name,
		   float value, float min, float max,
		   e_orientation direction);
	virtual ~EScrollBar();

	// Archiving
	EScrollBar(EMessage *from);
	virtual e_status_t Archive(EMessage *into, bool deep = true) const;
	static EArchivable *Instantiate(EMessage *from);

	void		SetValue(float value);
	float		Value() const;
	void		SetProportion(float ratio);
	float		Proportion() const;

	// Empty functions BEGIN --- just for derivative class
	virtual	void	ValueChanged(float value);
	// Empty functions END

	void		SetRange(float min, float max);
	void		GetRange(float *min, float *max) const;
	void		SetSteps(float smallStep, float largeStep);
	void		GetSteps(float *smallStep, float *largeStep) const;

	// SetTarget: If target isn't NULL, target->Ancestor() must equal to this->Ancestor().
	//            AKA. The function just be successful doing when it added to any parent.
	e_status_t	SetTarget(EView *target);
	EView		*Target() const;

	e_orientation	Orientation() const;

	virtual	void	Draw(ERect updateRect);
	virtual	void	MouseDown(EPoint where);
	virtual	void	MouseUp(EPoint where);
	virtual void	MouseMoved(EPoint where, euint32 code, const EMessage *a_message);
	virtual void	DetachedFromWindow();

private:
	friend class EWindow;
	friend class EView;

	e_orientation fOrientation;
	float fValue;
	float fRangeMin;
	float fRangeMax;
	float fStepSmall;
	float fStepLarge;
	EView *fTarget;

	bool fTracking;
	eint8 fTrackingState;
	EPoint fMousePosition;
	ERegion fTrackingRegion;
	void doScroll(eint8 state);

	EMessageRunner *fRunner;
};


#endif /* __cplusplus */

#endif /* __ETK_SCROLL_BAR_H__ */

