#ifndef __ETK_SCROLL_VIEW_H__
#define __ETK_SCROLL_VIEW_H__

#include <etk/interface/ScrollBar.h>

#ifdef __cplusplus /* Just for C++ */

class _IMPEXP_ETK EScrollView : public EView {
public:
	EScrollView(ERect frame, const char *name, EView *target,
		    euint32 resizingMode = E_FOLLOW_LEFT | E_FOLLOW_TOP,
		    euint32 flags = 0,
		    bool alwaysShowHorizontal = false,
		    bool alwaysShowVertical = false,
		    e_border_style border = E_FANCY_BORDER);
	virtual ~EScrollView();

	// Archiving
	EScrollView(EMessage *from);
	virtual e_status_t Archive(EMessage *into, bool deep = true) const;
	static EArchivable *Instantiate(EMessage *from);

	// SetTarget: When it return E_OK, the oldTarget was removed and destroy automatically.
	//            If you want to keep the oldTarget, try oldTarget->RemoveSelf() before.
	e_status_t	SetTarget(EView *newTarget);
	EView		*Target() const;

	virtual void	SetBorder(e_border_style border);
	e_border_style	Border() const;

	void		SetScrollBarAutoState(bool alwaysShowHorizontal, bool alwaysShowVertical);
	void		GetScrollBarAutoState(bool *alwaysShowHorizontal, bool *alwaysShowVertical) const;

	EScrollBar	*ScrollBar(e_orientation direction) const;

	virtual	void	SetFlags(euint32 flags); // auto-setting E_WILL_DRAW and E_FRAME_EVENTS
	virtual	void	Draw(ERect updateRect);
	virtual	void	FrameResized(float new_width, float new_height);

protected:
	virtual void	ChildRemoving(EView *child);

private:
	friend class EView;

	e_border_style fBorder;
	bool fAlwaysShowHorizontal;
	bool fAlwaysShowVertical;

	EScrollBar *fHSB;
	EScrollBar *fVSB;
	EView *fTarget;

	ERect TargetValidFrame(bool ignore_scrollbar = false) const;
};


#endif /* __cplusplus */

#endif /* __ETK_SCROLL_VIEW_H__ */

