/* --------------------------------------------------------------------------
 * 
 * ETK++ --- The Easy Toolkit for C++ programing
 * Copyright (C) 2004-2006, Anthony Lee, All Rights Reserved
 *
 * $COPYING
 * 
 * File: StatusBar.h
 * Description: EStatusBar --- A view that graphically indicates the progress
 * 
 * --------------------------------------------------------------------------*/

#ifndef __ETK_STATUS_BAR_H__
#define __ETK_STATUS_BAR_H__

#include <etk/interface/View.h>

#ifdef __cplusplus /* Just for C++ */

class _IMPEXP_ETK EStatusBar : public EView {
public:
	EStatusBar(ERect frame,
		   const char *name,
		   const char *label = NULL,
		   const char *trailing_label = NULL);
	virtual ~EStatusBar();

	// Archiving
	EStatusBar(EMessage *from);
	virtual e_status_t Archive(EMessage *into, bool deep = true) const;
	static EArchivable *Instantiate(EMessage *from);

	virtual void	SetBarHeight(float height);
	virtual void	SetText(const char *str);
	virtual	void	SetTrailingText(const char *str);
	virtual	void	SetMaxValue(float max);

	void		Update(float delta, const char *text = NULL, const char *trailing_text = NULL);
	void		Reset(const char *label = NULL, const char *trailing_label = NULL);
	virtual	void	SetTo(float value, const char *text = NULL, const char *trailing_text = NULL);

	float		CurrentValue() const;
	float		MaxValue() const;
	float		BarHeight() const;
	const char	*Text() const;
	const char	*TrailingText() const;
	const char	*Label() const;
	const char	*TrailingLabel() const;

	virtual void	MessageReceived(EMessage *msg);
	virtual	void	Draw(ERect updateRect);
	virtual	void	GetPreferredSize(float *width, float *height);

private:
	char *fLabel;
	char *fTrailingLabel;
	char *fText;
	char *fTrailingText;
	float fBarHeight;
	float fMaxValue;
	float fCurrentValue;
};

#endif /* __cplusplus */

#endif /* __ETK_STATUS_BAR_H__ */

