/* --------------------------------------------------------------------------
 * 
 * ETK++ --- The Easy Toolkit for C++ programing
 * Copyright (C) 2004-2006, Anthony Lee, All Rights Reserved
 *
 * $COPYING
 * 
 * File: StringView.h
 * Description: EStringView --- A view just for display a string
 * 
 * --------------------------------------------------------------------------*/

#ifndef __ETK_STRING_VIEW_H__
#define __ETK_STRING_VIEW_H__

#include <etk/support/StringArray.h>
#include <etk/interface/View.h>

#ifdef __cplusplus /* Just for C++ */

class _IMPEXP_ETK EStringView : public EView {
public:
	EStringView(ERect frame,
		    const char *name,
		    const char *initial_text,
		    euint32 resizeMode = E_FOLLOW_LEFT | E_FOLLOW_TOP,
		    euint32 flags = E_WILL_DRAW);
	virtual ~EStringView();

	// Archiving
	EStringView(EMessage *from);
	virtual e_status_t Archive(EMessage *into, bool deep = true) const;
	static EArchivable *Instantiate(EMessage *from);

	virtual void		SetText(const char *text);
	void			SetText(const EString &text);
	const char*		Text() const;

	virtual void		SetAlignment(e_alignment alignment);
	e_alignment		Alignment() const;

	virtual void		SetVerticalAlignment(e_vertical_alignment alignment);
	e_vertical_alignment	VerticalAlignment() const;

	virtual	void		Draw(ERect updateRect);
	virtual void		SetFont(const EFont *font, euint8 mask = E_FONT_ALL);
	virtual	void		GetPreferredSize(float *width, float *height);

private:
	EString fText;
	EStringArray *fTextArray;
	e_alignment fAlignment;
	e_vertical_alignment fVerticalAlignment;
};


inline void EStringView::SetText(const EString &text)
{
	SetText(text.String());
}


#endif /* __cplusplus */

#endif /* __ETK_STRING_VIEW_H__ */

