/* --------------------------------------------------------------------------
 *
 * ETK++ --- The Easy Toolkit for C++ programing
 * Copyright (C) 2004-2006, Anthony Lee, All Rights Reserved
 *
 * $COPYING
 *
 * File: TextView.h
 * Description: ETextView --- a multi-lines editable field
 * Warning: Unfinished
 *
 * --------------------------------------------------------------------------*/

#ifndef __ETK_TEXT_VIEW_H__
#define __ETK_TEXT_VIEW_H__

#include <etk/support/String.h>
#include <etk/interface/View.h>


typedef struct e_text_run {
	eint32			offset;		/* byte/character(utf8 mode) offset of first byte/character(utf8 mode) of run */
	e_font_desc		font;		/* font of run */
	e_rgb_color		color;		/* color of run */
	bool			underline;	/* whether to draw underline */
} e_text_run;


typedef struct e_text_run_array {
	eint32			count;		/* number of text runs */
	e_text_run		runs[1];	/* array of count number of runs */
} e_text_run_array;


#ifdef __cplusplus /* Just for C++ */


class EFile;


class _IMPEXP_ETK ETextView : public EView {
public:
	ETextView(ERect frame,
		  const char *name,
		  ERect textRect,
		  euint32 resizeMode = E_FOLLOW_LEFT | E_FOLLOW_TOP,
		  euint32 flags = E_WILL_DRAW | E_FRAME_EVENTS);
	ETextView(ERect frame,
		  const char *name,
		  ERect textRect,
		  const EFont *font, const e_rgb_color *color,
		  euint32 resizeMode = E_FOLLOW_LEFT | E_FOLLOW_TOP,
		  euint32 flags = E_WILL_DRAW | E_FRAME_EVENTS);
	virtual ~ETextView();

	const char	*Text() const;
	eint32		TextLength() const; // ASCII
	eint32		TextChars() const; // UTF-8

	char		ByteAt(eint32 index) const; // ASCII
	const char*	CharAt(eint32 index, euint8 *length = NULL) const; // UTF-8
	void		GetText(eint32 offset, eint32 length, char *buffer) const; // ASCII
	void		GetText(eint32 offset, eint32 length, char *buffer, eint32 buffer_size_in_bytes, bool utf8) const;

	void		SetText(const char *text, const e_text_run_array *runs = NULL, bool utf8 = false);
	void		SetText(const char *text, eint32 length, const e_text_run_array *runs = NULL, bool utf8 = false);
	void		SetText(EFile *file, eint64 fileOffset, eint32 length, const e_text_run_array *runs = NULL, bool utf8 = false);

	void		Insert(const char *text, const e_text_run_array *runs = NULL, bool utf8 = false);
	void		Insert(const char *text, eint32 length, const e_text_run_array *runs = NULL, bool utf8 = false);
	void		Insert(eint32 offset, const char *text, eint32 length, const e_text_run_array *runs = NULL, bool utf8 = false);
	void		Delete();
	void		Delete(eint32 startPos, eint32 endPos, bool utf8 = false); // exclude endPos

	virtual void	MakeEditable(bool editable);
	bool		IsEditable() const;

	virtual void	MakeSelectable(bool selectable);
	bool		IsSelectable() const;

	void		SetStylable(bool stylable);
	bool		IsStylable() const;

	void		SetMaxBytes(eint32 max);
	eint32		MaxBytes() const;

	void		SetTextRect(ERect textRect);
	ERect		TextRect() const;

	virtual void	Select(eint32 startPos, eint32 endPos, bool utf8 = false); // exclude endPos
	bool		GetSelection(eint32 *startPos, eint32 *endPos, bool utf8 = false) const; // exclude endPos
	void		SelectAll();
	void		Deselect();
	bool		IsSelected() const;

	eint32		CountLines() const;
	eint32		CurrentLine() const;
	void		GoToLine(eint32 index);

	eint32		LineAt(eint32 offset, bool utf8 = false) const;
	eint32		LineAt(EPoint pt, bool visible = true) const;
	EPoint		PointAt(eint32 offset, float *height = NULL, bool max_height = false, bool utf8 = false) const;
	eint32		OffsetAt(EPoint pt, bool visible = true, bool utf8 = false) const;
	eint32		OffsetAt(eint32 line, bool utf8 = false) const;

	float		LineWidth(eint32 lineIndex = 0) const;
	float		LineHeight(eint32 lineIndex = 0) const;
	float		TextHeight(eint32 fromLineIndex, eint32 toLineIndex) const;

	void		GetTextRegion(eint32 startPos, eint32 endPos, ERegion *region, bool utf8 = false) const; // exclude endPos
	virtual	void	ScrollToOffset(eint32 offset, bool utf8 = false);
	void		ScrollToSelection();

	void		SetTextBackground(e_rgb_color color);
	e_rgb_color	TextBackground() const;

	virtual	void	Draw(ERect updateRect);
	virtual	void	FrameResized(float new_width, float new_height);
	virtual	void	GetPreferredSize(float *width, float *height);

private:
	ERect fMargins;
	EString fText;
	e_text_run_array *fRunArray;

	bool fEditable;
	bool fSelectable;
	bool fStylable;
	eint32 fMaxBytes;

	eint32 fSelectStart;
	eint32 fSelectEnd;

	EList fLines;
	eint32 fCurrentLine;
	eint32 fCursor;

	e_rgb_color fTextBkColor;

	void ReScanRunArray(eint32 fromLine, eint32 toLine);
	void ReScanSize(eint32 fromLine, eint32 toLine);
	void ReScanLines();
};


inline void ETextView::SelectAll()
{
	Select(0, -1, false);
}


inline void ETextView::Deselect()
{
	Select(-1, 0, false);
}


inline bool ETextView::IsSelected() const
{
	return GetSelection(NULL, NULL, false);
}


inline void ETextView::Delete()
{
	eint32 startPos = 0, endPos = 0;
	if(GetSelection(&startPos, &endPos, false)) Delete(startPos, endPos, false);
}


#endif /* __cplusplus */

#endif /* __ETK_TEXT_VIEW_H__ */

