#ifndef __ETK_DEBUG_H__
#define __ETK_DEBUG_H__

#include <stdarg.h>
#include <stddef.h>
#include <stdlib.h>

#ifdef __cplusplus
#include <new>
#endif /* __cplusplus */

#include <etk/support/SupportDefs.h>

#if defined(__MWERKS__)
/* for Metrowerks, etc. */
#ifndef __FUNCTION__
#define __FUNCTION__	"Unknown"
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__	__FUNCTION__
#endif /* __PRETTY_FUNCTION__ */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef enum etk_debug_level {
	DEBUG_NORMAL,
	DEBUG_OUTPUT,
	DEBUG_WARNING,
	DEBUG_ERROR
} etk_debug_level;

_IMPEXP_ETK void etk_debug_log(etk_debug_level level, const char *format, va_list ap);

#if defined(ETK_DISABLE_MORE_CHECKS)
#	if __GNUC__ >= 3 || defined(__INTEL_COMPILER)
#		define ETK_DEBUG(...)				(void)0
#	else
#		ifdef __GNUC__
#			define ETK_DEBUG(a...)			(void)0
#		else
#			define ETK_DEBUG(a, b, c, d, e, f, g)	(void)0
#		endif
#	endif
#else
_IMPEXP_ETK void ETK_DEBUG(const char *format, ...);
#endif

_IMPEXP_ETK void ETK_OUTPUT(const char *format, ...);

#if defined(ETK_DISABLE_CHECKS)
#	if __GNUC__ >= 3 || defined(__INTEL_COMPILER)
#		define ETK_WARNING(...)				(void)0
#	else
#		ifdef __GNUC__
#			define ETK_WARNING(a...)		(void)0
#		else
#			define ETK_WARNING(a, b, c, d, e, f, g)	(void)0
#		endif
#	endif
#else
_IMPEXP_ETK void ETK_WARNING(const char *format, ...);
#endif

_IMPEXP_ETK void ETK_ERROR(const char *format, ...);

#ifdef ETK_BUILD_WITH_MEMORY_TRACING
_IMPEXP_ETK void* etk_calloc(size_t nmemb, size_t size, const char *file, int line, const char *method);
_IMPEXP_ETK void* etk_malloc(size_t size, const char *file, int line, const char *method);
_IMPEXP_ETK void* etk_realloc(void *ptr, size_t size, const char *file, int line, const char *method);
_IMPEXP_ETK void etk_free(void *ptr, const char *file, int line, const char *method);
#define calloc(a, b)	etk_calloc(a, b, __FILE__, __LINE__, "calloc")
#define malloc(a)	etk_malloc(a, __FILE__, __LINE__, "malloc")
#define realloc(a, b)	etk_realloc(a, b, __FILE__, __LINE__, "realloc")
#define free(a)		etk_free(a, __FILE__, __LINE__, "free")
#endif /* ETK_BUILD_WITH_MEMORY_TRACING */

#ifdef __cplusplus
} /* extern "C" */

#ifdef ETK_BUILD_WITH_MEMORY_TRACING
struct etk_memory_flag_t {};
_IMPEXP_ETK void* operator new(size_t size, const char *file, int line, const char *method, struct etk_memory_flag_t *flag);
_IMPEXP_ETK void* operator new[](size_t size, const char *file, int line, const char *method, struct etk_memory_flag_t *flag);
_IMPEXP_ETK void operator delete(void *ptr, const char *file, int line, const char *method, struct etk_memory_flag_t *flag);
_IMPEXP_ETK void operator delete[](void *ptr, const char *file, int line, const char *method, struct etk_memory_flag_t *flag);
#define new new(__FILE__, __LINE__, NULL, (struct etk_memory_flag_t*)0)
#endif /* ETK_BUILD_WITH_MEMORY_TRACING */

#endif /* __cplusplus */

#endif /* __ETK_DEBUG_H__ */

