#ifndef __ETK_PIXMAP_H__
#define __ETK_PIXMAP_H__

#include <etk/interface/Region.h>
#include <etk/render/Render.h>

#ifdef __cplusplus /* Just for C++ */

class _IMPEXP_ETK EPixmap : public ERender {
public:
	EPixmap();
	EPixmap(euint32 width, euint32 height, e_color_space space);
	EPixmap(ERect bounds, e_color_space space);
	virtual ~EPixmap();

	void*		Bits() const;
	euint32		BitsLength() const;
	euint32		BytesPerRow() const;
	e_color_space	ColorSpace() const;
	ERect		Bounds() const;
	void		MakeEmpty();

	bool		ResizeTo(euint32 width, euint32 height, e_color_space space);
	bool		ResizeTo(ERect bounds, e_color_space space);

	void		SetBits(const void *data, eint32 length, eint32 offset, e_color_space space);

	void		SetPixel(eint32 x, eint32 y, e_rgb_color color);
	e_rgb_color	GetPixel(eint32 x, eint32 y) const;

	void		DrawXPM(const char **xpm_data,
				eint32 destX, eint32 destY,
				eint32 srcX, eint32 srcY, eint32 srcW = -1, eint32 srcH = -1, euint8 alpha = 255);

private:
	void* fPtr;
	e_color_space fColorSpace;
	euint32 fRows;
	euint32 fColumns;
	euint32 fRowBytes;

	virtual void *AllocData(size_t size);
	virtual void FreeData(void *data);

	virtual e_status_t InitCheck() const;
	virtual void GetFrame(eint32 *originX, eint32 *originY, euint32 *width, euint32 *height) const;
	virtual void GetPixel(eint32 x, eint32 y, e_rgb_color &color) const;
	virtual void PutPixel(eint32 x, eint32 y, e_rgb_color color);
	virtual void PutRect(eint32 x, eint32 y, euint32 width, euint32 height, e_rgb_color color);
};

#endif /* __cplusplus */

#endif /* __ETK_PIXMAP_H__ */

