#ifndef __ETK_FILE_H__
#define __ETK_FILE_H__ 

#include <etk/storage/StorageDefs.h>
#include <etk/storage/Directory.h>

#ifdef __cplusplus /* Just for C++ */

class _IMPEXP_ETK EFile {
public:
	EFile();
	EFile(const char *path, euint32 open_mode, euint32 access_mode = E_USER_READ | E_USER_WRITE);
	EFile(const EEntry *entry, euint32 open_mode, euint32 access_mode = E_USER_READ | E_USER_WRITE);
	EFile(const EDirectory *dir, const char *leaf, euint32 open_mode, euint32 access_mode = E_USER_READ | E_USER_WRITE);
	EFile(const EFile &from);
	virtual ~EFile();

	e_status_t	InitCheck() const;
	e_status_t	SetTo(const char *path, euint32 open_mode, euint32 access_mode = E_USER_READ | E_USER_WRITE);
	e_status_t	SetTo(const EEntry *entry, euint32 open_mode, euint32 access_mode = E_USER_READ | E_USER_WRITE);
	e_status_t	SetTo(const EDirectory *dir, const char *leaf, euint32 open_mode, euint32 access_mode = E_USER_READ | E_USER_WRITE);
	void		Unset();

	bool		IsReadable() const;
	bool		IsWritable() const;

	ssize_t		Read(void *buffer, size_t size);
	ssize_t		ReadAt(eint64 pos, void *buffer, size_t size);
	ssize_t		Write(const void *buffer, size_t size);
	ssize_t		WriteAt(eint64 pos, const void *buffer, size_t size);

	eint64		Seek(eint64 position, euint32 seek_mode);
	eint64		Position() const;
	e_status_t	SetSize(eint64 size);

	EFile&		operator=(const EFile &from);

private:
	void *fFD;
	euint32 fMode;
};

#endif /* __cplusplus */

#endif /* __ETK_FILE_H__ */

