#ifndef __ETK_ERRORS_H__
#define __ETK_ERRORS_H__

#include <etk/ETKBuild.h>

/* Error baselines */
#define E_GENERAL_ERROR_BASE	E_MININT32
#define E_OS_ERROR_BASE		E_GENERAL_ERROR_BASE + 0x1000
#define E_APP_ERROR_BASE	E_GENERAL_ERROR_BASE + 0x2000
#define E_STORAGE_ERROR_BASE	E_GENERAL_ERROR_BASE + 0x3000

/* General Errors */
enum {
	E_NO_MEMORY = E_GENERAL_ERROR_BASE,
	E_IO_ERROR,
	E_PERMISSION_DENIED,
	E_BAD_INDEX,
	E_BAD_TYPE,
	E_BAD_VALUE,
	E_MISMATCHED_VALUES,
	E_NAME_NOT_FOUND,
	E_NAME_IN_USE,
	E_TIMED_OUT,
	E_INTERRUPTED,
	E_WOULD_BLOCK,
	E_CANCELED,
	E_NO_INIT,
	E_BUSY,
	E_NOT_ALLOWED,

	E_ERROR = -1,
	E_OK = 0,
	E_NO_ERROR = 0
};

/* Kernel Kit Errors */
enum {
	E_BAD_SEM_ID = E_OS_ERROR_BASE,
	E_NO_MORE_SEMS,

	E_BAD_THREAD_ID = E_OS_ERROR_BASE + 0x100,
	E_NO_MORE_THREADS,
	E_BAD_THREAD_STATE,
	E_BAD_TEAM_ID,
	E_NO_MORE_TEAMS,

	E_BAD_PORT_ID = E_OS_ERROR_BASE + 0x200,
	E_NO_MORE_PORTS,

	E_BAD_IMAGE_ID = E_OS_ERROR_BASE + 0x300,
	E_BAD_ADDRESS,
	E_NOT_AN_EXECUTABLE,
	E_MISSING_LIBRARY,
	E_MISSING_SYMBOL,

	E_DEBUGGER_ALREADY_INSTALLED = E_OS_ERROR_BASE + 0x400
};

/* Application Kit Errors */
enum {
	E_BAD_REPLY = E_APP_ERROR_BASE,
	E_DUPLICATE_REPLY,
	E_MESSAGE_TO_SELF,
	E_BAD_HANDLER,
	E_ALREADY_RUNNING,
};

/* Storage Kit Errors */
enum {
	E_FILE_ERROR = E_STORAGE_ERROR_BASE,
	E_ENTRY_NOT_FOUND,
	E_LINK_LIMIT,
};

#endif /* __ETK_ERRORS_H__ */

