#ifndef __ETK_LIST_H__
#define __ETK_LIST_H__

#include <etk/support/SupportDefs.h>

#ifdef __cplusplus /* Just for C++ */

class _IMPEXP_ETK EList {
public:
	EList(eint32 initialAllocSize = 0);
	EList(const EList &list);
	virtual ~EList();

	EList	&operator=(const EList &from);

	bool	AddItem(void *item);
	bool	AddItem(void *item, eint32 atIndex);
	bool	AddList(const EList *newItems);
	bool	AddList(const EList *newItems, eint32 atIndex);

	// RemoveItem(),RemoveItems(): the item WOULD NOT be destructed yet.
	bool	RemoveItem(void *item);
	void	*RemoveItem(eint32 index);
	bool	RemoveItems(eint32 index, eint32 count);

	// ReplaceItem(): the old item WOULD NOT be destructed yet.
	bool	ReplaceItem(eint32 index, void *newItem, void **oldItem = NULL);

	void	MakeEmpty();

	bool	SwapItems(eint32 indexA, eint32 indexB);
	bool	MoveItem(eint32 fromIndex, eint32 toIndex);

	void	SortItems(int (*cmp)(const void *a, const void *b));

	void	*ItemAt(eint32 index) const;
	void	*FirstItem() const;
	void	*LastItem() const;

	bool	HasItem(void *item) const;
	eint32	IndexOf(void *item) const;
	eint32	CountItems() const;
	bool	IsEmpty() const;

	void	DoForEach(bool (*func)(void *data));
	void	DoForEach(bool (*func)(void *data, void *user_data), void *user_data);

	// Items(): return the list, use it carefully please
	void	**Items() const;

private:
	void **Objects;

	eint32 ItemCount;
	eint32 ItemReal;

	bool Resize(eint32 count);
};

#endif /* __cplusplus */

#endif /* __ETK_LIST_H__ */

