#ifndef __ETK_SUPPORT_DEFS_H__
#define __ETK_SUPPORT_DEFS_H__

#include <string.h> /* for bzero */
#include <etk/ETKBuild.h>
#include <etk/support/Errors.h>

typedef eint32	e_status_t;
typedef eint64	e_bigtime_t;
typedef eint64	e_thread_id;
typedef euint32	e_type_code;

enum {
	E_ANY_TYPE 				= 'ANYT',
	E_BOOL_TYPE 				= 'BOOL',
	E_CHAR_TYPE 				= 'CHAR',
	E_DOUBLE_TYPE 				= 'DBLE',
	E_FLOAT_TYPE 				= 'FLOT',
	E_INT64_TYPE 				= 'LLNG',
	E_INT32_TYPE 				= 'LONG',
	E_INT16_TYPE 				= 'SHRT',
	E_INT8_TYPE 				= 'BYTE',
	E_MESSAGE_TYPE				= 'MSGG',
	E_MESSENGER_TYPE			= 'MSNG',
	E_POINTER_TYPE				= 'PNTR',
	E_SIZE_T_TYPE	 			= 'SIZT',
	E_SSIZE_T_TYPE	 			= 'SSZT',
	E_STRING_TYPE 				= 'CSTR',
	E_UINT64_TYPE				= 'ULLG',
	E_UINT32_TYPE				= 'ULNG',
	E_UINT16_TYPE 				= 'USHT',
	E_UINT8_TYPE 				= 'UBYT',
	E_POINT_TYPE				= 'SPNT',
	E_RECT_TYPE				= 'RECT',
	E_MIME_TYPE				= 'MIME'
};


#ifndef HAVE_BZERO
	#define bzero(ptr, len) memset(ptr, 0, len)
#endif /* HAVE_BZERO */

#ifndef __cplusplus

typedef	eint8	bool;

#ifndef false
#define false (0)
#endif

#ifndef true
#define true (!false)
#endif

#endif /* !__cplusplus */

#ifndef FALSE
#define FALSE (0)
#endif

#ifndef TRUE
#define TRUE (!FALSE)
#endif

#ifndef min_c
#define min_c(a, b)  ((a) > (b) ? (b) : (a))
#endif

#ifndef max_c
#define max_c(a, b)  ((a) > (b) ? (a) : (b))
#endif

#ifndef __cplusplus
	#ifndef min
	#define min(a, b) ((a) > (b) ? (b) : (a))
	#endif

	#ifndef max
	#define max(a, b) ((a) > (b) ? (a) : (b))
	#endif
#endif /* !__cplusplus */


#ifndef NULL
#  ifdef __cplusplus
#    define NULL        (0L)
#  else /* !__cplusplus */
#    define NULL        ((void*) 0)
#  endif /* !__cplusplus */
#endif


#ifdef ETK_OS_WIN32
#ifdef __GNUC__
#  ifndef _stdcall
#    define _stdcall  __attribute__((stdcall))
#  endif /* stdcall */
#endif /* __GNUC__ */
#endif /* ETK_OS_WIN32 */


/* We prefix variable declarations so they can
 * properly get exported in windows dlls or Metrowerks'.
 */
#ifndef _EXPORT
#  if defined(ETK_OS_WIN32) || defined(ETK_OS_CYGWIN) || defined(ETK_OS_BEOS)
#    define _EXPORT __declspec(dllexport)
#  else
#    define _EXPORT
#  endif
#endif /* _EXPORT */


#ifndef _IMPORT
#  if defined(ETK_OS_WIN32) || defined(ETK_OS_CYGWIN) || defined(ETK_OS_BEOS)
#    define _IMPORT __declspec(dllimport)
#  else
#    define _IMPORT
#  endif
#endif /* _IMPORT */


#ifndef _LOCAL
#  if __GNUC__ >= 3 && __GNUC_MINOR__ >= 3 && __GNUC_PATCHLEVEL__ > 3 && !defined(__MINGW32__)
#    define _LOCAL __attribute__((visibility("hidden")))
#  else
#    define _LOCAL
#  endif
#endif /* _LOCAL */


#ifdef ETK_COMPILATION
	#define _IMPEXP_ETK _EXPORT
#else /* !ETK_COMPILATION */
	#define _IMPEXP_ETK _IMPORT
#endif /* ETK_COMPILATION */


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

extern _IMPEXP_ETK const euint8 etk_major_version;
extern _IMPEXP_ETK const euint8 etk_minor_version;
extern _IMPEXP_ETK const euint8 etk_micro_version;
extern _IMPEXP_ETK const euint8 etk_interface_age;
extern _IMPEXP_ETK const euint8 etk_binary_age;

#ifdef __cplusplus
} /* extern "C" */
#endif /* __cplusplus */

#ifdef __cplusplus

#ifdef ETK_OS_WIN32
	#ifdef PostMessage
		#undef PostMessage
	#endif /* PostMessage */
	#ifdef SendMessage
		#undef SendMessage
	#endif /* SendMessage */
	#ifdef DispatchMessage
		#undef DispatchMessage
	#endif /* DispatchMessage */
	#ifdef CreateWindow
		#undef CreateWindow
	#endif /* CreateWindow */
#endif /* ETK_OS_WIN32 */

#endif /* __cplusplus */

#ifndef __ETK_DEBUG_H__
#include <etk/kernel/Debug.h>
#endif

#endif /* __ETK_SUPPORT_DEFS_H__ */

